from django.db import models

class TerritoryManager(models.Manager):
	
	def by_country_iso2(self, iso2 = None):
		return self.all() if iso2 == None else self.filter(country__iso2__iexact = iso2).order_by('abbr')

	def by_country_iso3(self, iso3 = None):
		return self.all() if iso3 == None else self.filter(country__iso3__iexact = iso3).order_by('abbr')

	def by_country_id(self, country_id = None):
		return self.all() if country_id == None else self.filter(country__id = country_id).order_by('abbr')



DEBUG = True
TEMPLATE_DEBUG = DEBUG

ADMINS = ()

MANAGERS = ADMINS

DATABASES = {
	'default': {
		'ENGINE': 'django.db.backends.sqlite3',
		'NAME': '/tmp/locality.db'
	}
}

TIME_ZONE = "UTC"

LANGUAGE_CODE = "en-us"

SITE_ID = 1

USE_I18N = True
USE_L10N = True

MEDIA_ROOT = ''
MEDIA_URL = ''
STATIC_ROOT = ''
STATIC_URL = '/static/'
ADMIN_MEDIA_PREFIX = '/static/admin/'

STATICFILES_DIRS = ()

STATICFILES_FINDERS = (
    'django.contrib.staticfiles.finders.FileSystemFinder',
    'django.contrib.staticfiles.finders.AppDirectoriesFinder',
    'django.contrib.staticfiles.finders.DefaultStorageFinder',
)

SECRET_KEY = 'd=eh)2xs8*px19*9_54-)q1=l=q*wy=(v+e002w95c@!)p_8)n'

TEMPLATE_LOADERS = (
    'django.template.loaders.filesystem.Loader',
    'django.template.loaders.app_directories.Loader',
    'django.template.loaders.eggs.Loader',
)

MIDDLEWARE_CLASSES = (
    'django.middleware.common.CommonMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
)

ROOT_URLCONF = 'locality.urls'

TEMPLATE_DIRS = ()

INSTALLED_APPS = (
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.sites',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'django.contrib.admin',
	'locality',
)

LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'handlers': {
        'mail_admins': {
            'level': 'ERROR',
            'class': 'django.utils.log.AdminEmailHandler'
        }
    },
    'loggers': {
        'django.request': {
            'handlers': ['mail_admins'],
            'level': 'ERROR',
            'propagate': True,
        },
    }
}



from django.core.validators import EMPTY_VALUES
from django.db.models import Q
from django.forms import fields, ValidationError
from django.utils.translation import ugettext_lazy as _

from locality.models import Country, Territory

class CountrySelectField(fields.Field):
	"""
	A form field that validates its input to iso2 or iso3 abbreviations
	or the name of countries in the database.
	"""
	default_error_messages = {
		'invalid': _("Select a Country."),
	}

	def clean(self, value):
		super(CountrySelectField, self).clean(value)

		if value in EMPTY_VALUES:
			return u''
		
		country = Country.objects.filter(Q(name__iexact=value) | Q(iso2__iexact=value) | Q(iso3__iexact=value))

		if not country.exists():
			raise ValidationError(self.default_error_messages['invalid'])

		return country[0]

class CountrySelectWidget(fields.Select):
	"""
	A select widget that uses all countries in alphabetical order as its choices.
	"""
	def __init__(self, attrs=None):
		choices = []
		for country in Country.objects.all().order_by('name'):
			choices.append((country.iso2, country.name,))
		super(CountrySelectWidget, self).__init__(attrs, choices=choices)



from setuptools import setup, find_packages

setup(
	name = "django-locality",
	version = "0.1.1",
	url = 'https://github.com/rfkrocktk/django-locality',
	license = 'AGPL',
	description = 'Countries and territories, made usable.',
	author = 'TK Kocheran',
	author_email = 'rfkrocktk@gmail.com',
	packages = find_packages('src'),
	package_dir = {'': 'src'},
	package_data = {
		'': ['locality/fixtures/*.json']
	},
	install_requires = [
		'Django>=1.3.0',
		'setuptools'
	],
)



from django.core import serializers
from django.http import HttpResponse, Http404
from django.shortcuts import get_object_or_404
from django.utils.encoding import force_unicode
from django.utils.functional import Promise

from locality.models import Country, Territory

import simplejson as json

def countries(request, to = "json", mimetype = "application/json"):
	return HttpResponse(serializers.serialize(to, Country.objects.all()), mimetype="%s; charset=utf-8" % mimetype)

def territories(request, to = "json", mimetype = "application/json"):
	return HttpResponse(serializers.serialize(to, Territory.objects.all()), mimetype="%s; charset=utf-8" % mimetype)

def territories_by_country_iso2(request, iso2 = None, to = "json", mimetype = "application/json"):
	return HttpResponse(serializers.serialize(to, Territory.objects.by_country_iso2(iso2)), mimetype="%s; charset=utf-8" % mimetype)

def territories_by_country_iso3(request, iso3 = None, to = "json", mimetype = "application/json"):
	return HttpResponse(serializers.serialize(to, Territory.objects.by_country_iso3(iso3)), mimetype="%s; charset=utf-8" % mimetype)

def territories_by_country_id(request, country_id = None, to = "json", mimetype = "application/json"):
	return HttpResponse(serializers.serialize(to, Territory.objects.by_country_id(country_id)), mimetype="%s; charset=utf-8" % mimetype)

def countries(request, to = "json", mimetype = "application/json"):
	return HttpResponse(serializers.serialize(to, Country.objects.all()), mimetype="%s; charset=utf-8" % mimetype)

def country_by_iso2(request, iso2 = None, to = "json", mimetype = "application/json"):
	return HttpResponse(serializers.serialize(to, get_object_or_404(Country, iso2__iexact = iso2)), mimetype="%s; charset=utf-8" % mimetype)

def country_by_iso3(request, iso3 = None, to = "json", mimetype = "application/json"):
	return HttpResponse(serializers.serialize(to, get_object_or_404(Country, iso3__iexact = iso3)), mimetype="%s; charset=utf-8" % mimetype)

def country_by_id(request, country_id = None, to = "json", mimetype = "application/json"):
	return HttpResponse(serializers.serialize(to, get_object_or_404(Country, pk = country_id)), mimetype="%s; charset=utf-8" % mimetype)

def generate_territories(request):
	from django.contrib.localflavor.ar import ar_provinces
	from django.contrib.localflavor.au import au_states
	from django.contrib.localflavor.at import at_states
	from django.contrib.localflavor.be import be_provinces
	from django.contrib.localflavor.br import br_states
	from django.contrib.localflavor.ca import ca_provinces
	from django.contrib.localflavor.de import de_states
	from django.contrib.localflavor.nl import nl_provinces
	from django.contrib.localflavor.in_ import in_states
	from django.contrib.localflavor.it import it_province
	from django.contrib.localflavor.mx import mx_states
	from django.contrib.localflavor.es import es_provinces
	from django.contrib.localflavor.ch import ch_states
	from django.contrib.localflavor.tr import tr_provinces
	from django.contrib.localflavor.us import us_states
	
	counter = SuperCounter(start = 1)
	c = counter

	output = []
	output.extend(create_territories("AR", ar_provinces.PROVINCE_CHOICES, c))
	output.extend(create_territories("AU", au_states.STATE_CHOICES, c))
	output.extend(create_territories("AT", at_states.STATE_CHOICES, c))
	output.extend(create_territories("BE", be_provinces.PROVINCE_CHOICES, c))
	output.extend(create_territories("BR", br_states.STATE_CHOICES, c))
	output.extend(create_territories("CA", ca_provinces.PROVINCE_CHOICES, c))
	output.extend(create_territories("DE", de_states.STATE_CHOICES, c))
	output.extend(create_territories("NL", nl_provinces.PROVINCE_CHOICES, c))
	output.extend(create_territories("IN", in_states.STATE_CHOICES, c))
	output.extend(create_territories("IT", it_province.PROVINCE_CHOICES, c))
	output.extend(create_territories("MX", mx_states.STATE_CHOICES, c))
	output.extend(create_territories("ES", es_provinces.PROVINCE_CHOICES, c))
	output.extend(create_territories("CH", ch_states.STATE_CHOICES, c))
	output.extend(create_territories("TR", tr_provinces.PROVINCE_CHOICES, c))
	output.extend(create_territories("US", us_states.STATE_CHOICES, c))

	return HttpResponse(LazyEncoder(indent="\t").encode(output), mimetype="application/json; charset=utf-8")
	
def create_territories(country_abbr, territories, counter):
	result = []
	country = Country.objects.get(iso2=country_abbr)
	
	for territory in territories:
		result.append({ 
		'model': "locality.territory",
		'pk': counter.get(),
		'fields': {
			'abbr': territory[0], 
			'name': territory[1], 
			'country': country.id 
		}})

		counter.increment()

	return result

class LazyEncoder(json.JSONEncoder):
	def default(self, obj):
		if isinstance(obj, Promise):
			return force_unicode(obj)
		return super(LazyEncoder, self).default(obj)

class SuperCounter():
	__current__ = 0

	def __init__(self, start = 0):
		self.__current__ = start

	def get(self):
		return self.__current__

	def increment(self):
		self.__current__ = self.__current__ + 1
		return self.__current__



from django.conf.urls.defaults import patterns, include, url
from django.contrib import admin
admin.autodiscover()

urlpatterns = patterns('locality.views',
	# generate initial data from localflavors for territories
	url(r'^json/generate/territories/?$', 'generate_territories'),
	# get all countries
	url(r'^json/countries/(?:all/?)?$', 'countries'),
	url(r'^xml/countries/(?:all/?)?$', 'countries', { 'to': 'xml', 'mimetype': 'application/xml' }),
	url(r'^yaml/countries/(?:all/?)?$', 'countries', { 'to': 'yaml', 'mimetype': 'application/yaml' }),
	# get all territories
	url(r'^json/territories/(?:all/?)?$', 'territories'),
	# get json of territories either globally or by country
	url(r'^json/territories(?:/(?P<iso2>\w{2})/?)?$', 'territories_by_country_iso2'),
	url(r'^json/territories(?:/(?P<iso3>\w{3})/?)?$', 'territories_by_country_iso3'),
	url(r'^json/territories(?:/(?P<country_id>\d+)/?)?$', 'territories_by_country_id'),
	# get xml of territories either globally or by country
	url(r'^xml/territories(?:/(?P<iso2>\w{2})/?)?$', 'territories_by_country_iso2', { 'to': 'xml', 'mimetype': 'text/xml' }),
	url(r'^xml/territories(?:/(?P<iso3>\w{3})/?)?$', 'territories_by_country_iso3', { 'to': 'xml', 'mimetype': 'text/xml' }),
	url(r'^xml/territories(?:/(?P<country_id>\d+)/?)?$', 'territories_by_country_id', { 'to': 'xml', 'mimetype': 'text/xml' }),
	# get yaml of territories either globally or by country
	url(r'^yaml/territories(?:/(?P<iso2>\w{2})/?)?$', 'territories_by_country_iso2', { 'to': 'yaml', 'mimetype': 'text/yaml' }),
	url(r'^yaml/territories(?:/(?P<iso3>\w{3})/?)?$', 'territories_by_country_iso3', { 'to': 'yaml', 'mimetype': 'text/yaml' }),
	url(r'^yaml/territories(?:/(?P<country_id>\d+)/?)?$', 'territories_by_country_id', { 'to': 'yaml', 'mimetype': 'text/yaml' }),
	# get json of country
	url(r'^json/country/(?P<iso2>\w{2})/?$', 'country_by_iso2'),
	url(r'^json/country/(?P<iso3>\w{3})/?$', 'country_by_iso3'),
	url(r'^json/country/(?P<country_id>\d+)/?$', 'country_by_id'),
   	# get xml of country
 	url(r'^xml/country/(?P<iso2>\w{2})/?$', 'country_by_iso2', { 'to': 'xml', 'mimetype': 'application/xml' }),
	url(r'^xml/country/(?P<iso3>\w{3})/?$', 'country_by_iso3', { 'to': 'xml', 'mimetype': 'application/xml' }),
	url(r'^xml/country/(?P<country_id>\d+)/?$', 'country_by_id', { 'to': 'xml', 'mimetype': 'application/xml' }),
	# get yaml of country
	url(r'^yaml/country/(?P<iso2>\w{2})/?$', 'country_by_iso2', { 'to': 'yaml', 'mimetype': 'application/yaml' }),
	url(r'^yaml/country/(?P<iso3>\w{3})/?$', 'country_by_iso3', { 'to': 'yaml', 'mimetype': 'application/yaml' }),
	url(r'^yaml/country/(?P<country_id>\d+)/?$', 'country_by_id', { 'to': 'yaml', 'mimetype': 'application/yaml' }),
#  	url(r'^admin/', include(admin.site.urls)),
)



from django.contrib import admin
from django.db import models
from django.utils.translation import ugettext_lazy as _

from locality import managers

class Country(models.Model):
	iso2 = models.CharField('ISO 3166-1 Alpha 2 Name', max_length=2, unique=True)
	iso3 = models.CharField('ISO 3166-1 Alpha 3 Name', max_length=3, unique=True)
	name = models.CharField('Country Name', max_length=32, unique=True)
	
	def __str__(self):
		return self.name

	def __unicode__(self):
		return self.name

	@property
	def abbr(self):
		return self.iso2

	class Meta:
		verbose_name = _('Country')
		verbose_name_plural = _('Countries')
		ordering = ('iso2', 'name',)

class CountryAdmin(admin.ModelAdmin):
	list_display = ('iso2', 'iso3', 'name',)


class Territory(models.Model):
	abbr = models.CharField('Territory Abbreviation', max_length=5)
	name = models.CharField('Territory Name', max_length=32)
	country = models.ForeignKey('Country', related_name="territories", on_delete=models.CASCADE)
	
	objects = managers.TerritoryManager()

	def __str__(self):
		return "%s, %s" % (self.name, self.country.name)

	def __unicode__(self):
		return u"%s, %s" % (self.name, self.country.name)

	class Meta:
		verbose_name = _('Territory')
		verbose_name_plural = _('Territories')
		ordering = ('abbr', 'name',)


class TerritoryAdmin(admin.ModelAdmin):
	list_display = ('abbr', 'country', 'name',)


admin.site.register(Country, CountryAdmin)
admin.site.register(Territory, TerritoryAdmin)
